"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = runRules;

var _actions = require("./actions");

var _actions2 = _interopRequireDefault(_actions);

var _deepcopy = require("deepcopy");

var _deepcopy2 = _interopRequireDefault(_deepcopy);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toExecutor(predicate, action) {
  return function (schema, uiSchema, formData) {
    predicate.run(formData).then(function (events) {
      events.forEach(function (_ref) {
        var params = _ref.params;
        return action(params, schema, uiSchema, formData);
      });
      return { schema: schema, uiSchema: uiSchema, formData: formData };
    });
  };
}

function runRules() {
  var formData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _ref2 = arguments[1];
  var rulesEngine = _ref2.rulesEngine,
      rules = _ref2.rules,
      schema = _ref2.schema,
      _ref2$uiSchema = _ref2.uiSchema,
      uiSchema = _ref2$uiSchema === undefined ? {} : _ref2$uiSchema,
      _ref2$extraActions = _ref2.extraActions,
      extraActions = _ref2$extraActions === undefined ? {} : _ref2$extraActions;

  var ruleExecutors = rules.map(function (rule) {
    var predicate = new rulesEngine([], schema);
    predicate.addRule(rule);
    var action = (0, _actions2.default)(rule, schema, uiSchema, extraActions);
    return toExecutor(predicate, action);
  });

  var schemaCopy = (0, _deepcopy2.default)(schema);
  var uiSchemaCopy = (0, _deepcopy2.default)(uiSchema);
  var formDataCopy = (0, _deepcopy2.default)(formData);

  var execute = ruleExecutors.reduce(function (agg, executor) {
    return agg.then(function () {
      return executor(schemaCopy, uiSchemaCopy, formDataCopy);
    });
  }, Promise.resolve({}));
  return execute.then(function () {
    return {
      schema: schemaCopy,
      uiSchema: uiSchemaCopy,
      formData: formDataCopy
    };
  });
}